﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HEAL.Attic;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {

  [Item("SwapEquipmentManipluator", "Swaps two equipments by placing each in the location of the other.")]
  [StorableType("FC7FD665-84A3-4BEA-A139-8248120375E0")]
  public class SwapEquipmentManipluator : GQAPManipulator {

    [StorableConstructor]
    protected SwapEquipmentManipluator(StorableConstructorFlag _) : base(_) { }
    protected SwapEquipmentManipluator(SwapEquipmentManipluator original, Cloner cloner) : base(original, cloner) { }
    public SwapEquipmentManipluator()
      : base() {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SwapEquipmentManipluator(this, cloner);
    }

    public static void Apply(IRandom random, IntegerVector assignment) {
      if (assignment.Length < 2) throw new InvalidOperationException("Vector is too small for swap operation.");
      int equipment1 = random.Next(assignment.Length), equipment2 = random.Next(assignment.Length);
      int counter = 1;
      while (assignment[equipment1] == assignment[equipment2] && counter < assignment.Length) {
        equipment2 = random.Next(assignment.Length);
        counter++;
      }
      int location = assignment[equipment1];
      assignment[equipment1] = assignment[equipment2];
      assignment[equipment2] = location;
    }

    protected override void Manipulate(IRandom random, IntegerVector vector, GQAPInstance problemInstance) {
      Apply(random, vector);
    }
  }
}
